@@ -106,8 +106,6 @@ Also, the directory tests/ contains several examples of well-formed "Execute" re
 * The files PMLExecuteRequest*.xml contain requests that can be submitted to the live PML WPS service.
 """
 
-from __future__ import (absolute_import, division, print_function)
-
 from owslib.etree import etree
 from owslib.ows import DEFAULT_OWS_NAMESPACE, XLINK_NAMESPACE
 from owslib.ows import ServiceIdentification, ServiceProvider, OperationsMetadata, BoundingBox
@@ -116,12 +114,9 @@ from owslib.util import (testXMLValue, testXMLAttribute, build_get_url, clean_ow
                          getNamespace, element_to_string, nspath, openURL, nspath_eval, log, Authentication)
 from xml.dom.minidom import parseString
 from owslib.namespaces import Namespaces
-try:                    # Python 3
-    from urllib.parse import urlparse
-except ImportError:     # Python 2
-    from urlparse import urlparse
+from urllib.parse import urlparse
+import warnings
 
-import six
 # namespace definition
 n = Namespaces()
 
@@ -171,15 +166,7 @@ def is_literaldata(val):
     """
     Checks if the provided value is a string (includes unicode).
     """
-    is_str = isinstance(val, str)
-    if not is_str:
-        # on python 2.x we need to check unicode
-        try:
-            is_str = isinstance(val, unicode)  # noqa: F821
-        except Exception:
-            # unicode is not available on python 3.x
-            is_str = False
-    return is_str
+    return isinstance(val, str)
 
 
 def is_boundingboxdata(val):
@@ -196,6 +183,24 @@ def is_complexdata(val):
     return isinstance(val, IComplexDataInput)
 
 
+def _fix_auth(auth, username=None, password=None, verify=None, cert=None):
+    """Updates auth from deprecated parameters username, password, verify and cert."""
+    if any(p is not None for p in (username, password, verify, cert)):
+        message = 'The use of "username", "password", "verify", and "cert" is deprecated. ' + \
+                  'Please use the "auth" keyword during class instantiation. ' + \
+                  'These keywords will be removed in a future release.'
+        warnings.warn(message, DeprecationWarning)
+    if username is not None:
+        auth.username = username
+    if password is not None:
+        auth.password = password
+    if verify is not None:
+        auth.verify = verify
+    if cert is not None:
+        auth.cert = cert
+    return auth
+
+
 class IComplexDataInput(object):
 
     """
@@ -219,23 +224,16 @@ class WebProcessingService(object):
     """
 
     def __init__(self, url, version=WPS_DEFAULT_VERSION, username=None, password=None, verbose=False, skip_caps=False,
-                 headers=None, verify=True, cert=None, timeout=None, auth=None):
+                 headers=None, verify=None, cert=None, timeout=None, auth=None):
         """
         Initialization method resets the object status.
         By default it will execute a GetCapabilities invocation to the remote service,
         which can be skipped by using skip_caps=True.
-        """
 
-        if auth:
-            if username:
-                auth.username = username
-            if password:
-                auth.password = password
-            if cert:
-                auth.cert = cert
-            if verify:
-                auth.verify = verify
-        self.auth = auth or Authentication(username, password, cert, verify)
+        Parameters username, password, verify and cert are deprecated. Please use auth parameter.
+        """
+        self.auth = auth or Authentication()
+        _fix_auth(self.auth, username, password, verify, cert)
 
         # fields passed in from object initializer
         self.url = clean_ows_url(url)
@@ -301,8 +299,11 @@ class WebProcessingService(object):
 
         if identifier == 'all':
             return processes
-        else:
-            return processes[0]
+        # return process with given identifier
+        for process in processes:
+            if process.identifier == identifier:
+                return process
+        raise ValueError('process with identifier {} not found'.format(identifier))
 
     def execute(self, identifier, inputs, output=None, mode=ASYNC, lineage=False, request=None, response=None):
         """
@@ -465,14 +466,8 @@ class WPSReader(object):
         Method to get and parse a WPS document, returning an elementtree instance.
         :param str url: WPS service base url.
         :param str data: GET: dictionary of HTTP (key, value) parameter pairs, POST: XML document to post
-        :param str username: optional user credentials
-        :param str password: optional user credentials
         """
-        username = username or self.auth.username
-        password = password or self.auth.password
-        cert = cert or self.auth.cert
-        verify = verify or self.auth.verify
-
+        _fix_auth(self.auth, username, password, verify, cert)
         if method == 'Get':
             # full HTTP request url
             request_url = build_get_url(url, data, overwrite=True)
@@ -481,14 +476,14 @@ class WPSReader(object):
             # split URL into base url and query string to use utility function
             spliturl = request_url.split('?')
             u = openURL(spliturl[0], spliturl[
-                        1], method='Get', username=username, password=password,
-                        headers=headers, verify=verify, cert=cert, timeout=self.timeout)
+                        1], method='Get', username=self.auth.username, password=self.auth.password,
+                        headers=headers, verify=self.auth.verify, cert=self.auth.cert, timeout=self.timeout)
             return etree.fromstring(u.read())
 
         elif method == 'Post':
             u = openURL(url, data, method='Post',
-                        username=username, password=password,
-                        headers=headers, verify=verify, cert=cert, timeout=timeout)
+                        username=self.auth.username, password=self.auth.password,
+                        headers=headers, verify=self.auth.verify, cert=self.auth.cert, timeout=timeout)
             return etree.fromstring(u.read())
 
         else:
@@ -517,13 +512,11 @@ class WPSCapabilitiesReader(WPSReader):
             version=version, verbose=verbose, timeout=timeout, auth=auth)
 
     def readFromUrl(self, url, username=None, password=None,
-                    headers=None, verify=True, cert=None):
+                    headers=None, verify=None, cert=None):
         """
         Method to get and parse a WPS capabilities document, returning an elementtree instance.
 
         :param str url: WPS service base url, to which is appended the HTTP parameters: service, version, and request.
-        :param str username: optional user credentials
-        :param str password: optional user credentials
         """
         return self._readFromUrl(url,
                                  {'service': 'WPS', 'request':
@@ -545,7 +538,7 @@ class WPSDescribeProcessReader(WPSReader):
             version=version, verbose=verbose, timeout=timeout, auth=auth)
 
     def readFromUrl(self, url, identifier, username=None, password=None,
-                    headers=None, verify=True, cert=None):
+                    headers=None, verify=None, cert=None):
         """
         Reads a WPS DescribeProcess document from a remote service and returns the XML etree object
 
@@ -571,7 +564,7 @@ class WPSExecuteReader(WPSReader):
         super(WPSExecuteReader, self).__init__(verbose=verbose, timeout=timeout, auth=auth)
 
     def readFromUrl(self, url, data={}, method='Get', username=None, password=None,
-                    headers=None, verify=True, cert=None):
+                    headers=None, verify=None, cert=None):
         """
         Reads a WPS status document from a remote service and returns the XML etree object.
         :param str url: the URL to submit the GET/POST request to.
@@ -588,24 +581,15 @@ class WPSExecution(object):
     """
 
     def __init__(self, version=WPS_DEFAULT_VERSION, url=None, username=None, password=None, verbose=False,
-                 headers=None, verify=True, cert=None, timeout=None, auth=None):
-
-        if auth:
-            if username:
-                auth.username = username
-            if password:
-                auth.password = password
-            if cert:
-                auth.cert = cert
-            if verify:
-                auth.verify = verify
+                 headers=None, verify=None, cert=None, timeout=None, auth=None):
 
         # initialize fields
         self.url = url
         self.version = version
         self.verbose = verbose
         self.headers = headers
-        self.auth = auth or Authentication(username, password, cert, verify)
+        self.auth = auth or Authentication()
+        _fix_auth(self.auth, username, password, verify, cert)
         self.timeout = timeout
 
         # request document
@@ -863,7 +847,7 @@ class WPSExecution(object):
                     if filepath is None:
                         filepath = 'wps.out'
                     for data in output.data:
-                        content = content + data
+                        content = content + data.encode()
 
             # write out content
             if content != '':
@@ -1301,7 +1285,7 @@ class Output(InputOutput):
             xlinkns = get_namespaces()['xlink']
             xlink_href = '{{{}}}href'.format(xlinkns)
 
-            if xlink_href in referenceElement.keys():
+            if xlink_href in list(referenceElement.keys()):
                 self.reference = referenceElement.get(xlink_href)
             else:
                 self.reference = referenceElement.get('href')
@@ -1396,7 +1380,7 @@ class Output(InputOutput):
         """
         url = self.reference
         if url is None:
-            return ""
+            return b''
 
         # a) 'http://cida.usgs.gov/climate/gdp/process/RetrieveResultServlet?id=1318528582026OUTPUT.601bb3d0-547f-4eab-8642-7c7d2834459e'  # noqa
         # b) 'http://rsg.pml.ac.uk/wps/wpsoutputs/outputImage-11294Bd6l2a.tif'
@@ -1909,24 +1893,24 @@ def printInputOutput(value, indent=''):
     '''
 
     # InputOutput fields
-    print('%s identifier=%s, title=%s, abstract=%s, data type=%s' %
-          (indent, value.identifier, value.title, value.abstract, value.dataType))
+    print(('{} identifier={}, title={}, abstract={}, data type={}'.format(
+          indent, value.identifier, value.title, value.abstract, value.dataType)))
     for val in value.allowedValues:
-        print('%s Allowed Value: %s' % (indent, printValue(val)))
+        print(('{} Allowed Value: {}'.format(indent, printValue(val))))
     if value.anyValue:
         print(' Any value allowed')
     for val in value.supportedValues:
-        print('%s Supported Value: %s' % (indent, printValue(val)))
-    print('%s Default Value: %s ' % (indent, printValue(value.defaultValue)))
+        print(('{} Supported Value: {}'.format(indent, printValue(val))))
+    print(('{} Default Value: {} '.format(indent, printValue(value.defaultValue))))
 
     # Input fields
     if isinstance(value, Input):
-        print('%s minOccurs=%d, maxOccurs=%d' %
-              (indent, value.minOccurs, value.maxOccurs))
+        print(('{} minOccurs={}, maxOccurs={}'.format(
+              indent, value.minOccurs, value.maxOccurs)))
 
     # Output fields
     if isinstance(value, Output):
-        print('%s reference=%s, mimeType=%s' %
-              (indent, value.reference, value.mimeType))
+        print(('{} reference={}, mimeType={}'.format(
+              indent, value.reference, value.mimeType)))
         for datum in value.data:
-            print('%s Data Value: %s' % (indent, printValue(datum)))
+            print(('{} Data Value: {}'.format(indent, printValue(datum))))
